%define libmajor	2
%define libname		%mklibname %{name} %{libmajor}

%define irmlmajor	1
%define irmllib		%mklibname irml %{irmlmajor}

%define devname		%mklibname %{name} -d

Summary:	Library to abstract low-level threading details
Name:		tbb
Version:	2020.3
Release:	%mkrel 4
License:	ASL2.0
Group:		Development/C++
URL:		http://threadingbuildingblocks.org/

Source0:	https://github.com/intel/tbb/archive/v%{version}/%{name}-%{version}.tar.gz
# These are downstream sources.
Source6:	tbb.pc
Source7:	tbbmalloc.pc
Source8:	tbbmalloc_proxy.pc

# (Fedora) Don't snip -Wall from C++ flags.  Add -fno-strict-aliasing, as that
# uncovers some static-aliasing warnings.
# Related: https://bugzilla.redhat.com/show_bug.cgi?id=1037347
Patch0: tbb-2019-dont-snip-Wall.patch

# Make attributes of aliases match those on the aliased function.
Patch1: tbb-2020-attributes.patch

# Fix test-thread-monitor, which had multiple bugs that could (and did, on
# ppc64le) result in a hang.
Patch2: tbb-2019-test-thread-monitor.patch

# Fix a test that builds a 4-thread barrier, but cannot guarantee that more
# than 2 threads will be available to use it.
Patch3: tbb-2019-test-task-scheduler-init.patch

# Fix compilation on aarch64 and s390x.  See
# https://github.com/intel/tbb/issues/186
Patch4: tbb-2019-fetchadd4.patch

BuildRequires: doxygen
BuildRequires: gcc-c++
BuildRequires: libgomp-devel
BuildRequires: python3-devel
BuildRequires: swig

%description
Threading Building Blocks (TBB) is a C++ runtime library that
abstracts the low-level threading details necessary for optimal
multi-core performance.  It uses common C++ templates and coding style
to eliminate tedious threading implementation work.

TBB requires fewer lines of code to achieve parallelism than other
threading models.  The applications you write are portable across
platforms.  Since the library is also inherently scalable, no code
maintenance is required as more processor cores become available.

%package -n %{libname}
Summary:	Library to abstract low-level threading details
Group:		System/Libraries
Obsoletes:	%{name} < 4.1-1.20121003.1

%description -n %{libname}
Threading Building Blocks (TBB) is a C++ runtime library that
abstracts the low-level threading details necessary for optimal
multi-core performance.  It uses common C++ templates and coding style
to eliminate tedious threading implementation work.

TBB requires fewer lines of code to achieve parallelism than other
threading models.  The applications you write are portable across
platforms.  Since the library is also inherently scalable, no code
maintenance is required as more processor cores become available.

%package -n %{irmllib}
Summary:        Library to abstract low-level threading details
Group:          System/Libraries

%description -n %{irmllib}
Threading Building Blocks (TBB) is a C++ runtime library that
abstracts the low-level threading details necessary for optimal
multi-core performance.  It uses common C++ templates and coding style
to eliminate tedious threading implementation work.

TBB requires fewer lines of code to achieve parallelism than other
threading models.  The applications you write are portable across
platforms.  Since the library is also inherently scalable, no code
maintenance is required as more processor cores become available.

%package -n %{devname}
Summary:	The Threading Building Blocks C++ headers and shared development libraries
Group:		Development/C++
Requires:	%{libname} = %{version}-%{release}
Requires:	%{irmllib} = %{version}-%{release}
Provides:	%{name}-devel = %{version}-%{release}
Obsoletes:	%{name}-devel < 4.1-1.20121003.1

%description -n %{devname}
Header files and shared object symlinks for the Threading Building
Blocks (TBB) C++ libraries.


%package doc
Summary: The Threading Building Blocks documentation
Group: Documentation
Provides: bundled(jquery)

%description doc
PDF documentation for the user of the Threading Building Block (TBB)
C++ library.


%package -n python3-%{name}
Summary: Python 3 TBB module
Group: Development/Python
%{?python_provide:%python_provide python3-%{name}}

%description -n python3-%{name}
Python 3 TBB module.


%prep
%autosetup -p1 -n oneTBB-%{version}

# For repeatable builds, don't query the hostname or architecture
sed -i 's/"`hostname -s`" ("`uname -m`"/%{_vendor} buildsystem (%{_arch}/' \
    build/version_info_linux.sh

# Do not assume the RTM instructions are available.
# Insert --as-needed before the libraries to be linked.
sed -e 's/-mrtm//' \
    -e "s/-fPIC/& -Wl,--as-needed/" \
    -i build/linux.gcc.inc

# Invoke the right python binary directly
sed -i 's,env python,python3,' python/TBB.py python/tbb/__*.py

# Remove shebang from files that don't need it
sed -i '/^#!/d' python/tbb/{pool,test}.py

# Fix libdir on 64-bit systems
if [ "%{_libdir}" != "%{_prefix}/lib" ]; then
  sed -i.orig 's/"lib"/"%{_lib}"/' cmake/TBBMakeConfig.cmake
  touch -r cmake/TBBMakeConfig.cmake.orig cmake/TBBMakeConfig.cmake
  rm cmake/TBBMakeConfig.cmake.orig
fi

%build
compiler=""
if [[ %{__cc} == *"gcc"* ]]; then
    compiler="gcc"
elif [[ %{__cc} == *"clang"* ]]; then
    compiler="clang"
else
    compiler="%{__cc}"
fi

%make_build tbb_build_prefix=obj stdver=c++14 \
    compiler=${compiler} \
    CXXFLAGS="%{optflags} -DDO_ITT_NOTIFY -DUSE_PTHREAD" \
    LDFLAGS="$RPM_LD_FLAGS -lpthread"
for file in %{SOURCE6} %{SOURCE7} %{SOURCE8}; do
    base=$(basename ${file})
    sed 's/_FEDORA_VERSION/%{version}/' ${file} > ${base}
    touch -r ${file} ${base}
done

# Build for python 3
. build/obj_release/tbbvars.sh
pushd python
%make_build -C rml stdver=c++14 \
    compiler=${compiler} \
    CPLUS_FLAGS="%{optflags} -DDO_ITT_NOTIFY -DUSE_PTHREAD" \
    LDFLAGS="$RPM_LD_FLAGS -lpthread"
cp -p rml/libirml.so* .
%py3_build
popd

# Build the documentation
make doxygen

%check
# This test assumes it can create thread barriers for arbitrary numbers of
# threads, but tbb limits the number of threads spawned to a function of the
# number of CPUs available.  Some of the koji builders have a small number of
# CPUs, so the test hangs waiting for threads that have not been created to
# arrive at the barrier.  Skip this test until upstream fixes it.
sed -i '/test_task_scheduler_observer/d' build/Makefile.test

make test tbb_build_prefix=obj stdver=c++14 CXXFLAGS="%{optflags}"

%install
mkdir -p $RPM_BUILD_ROOT/%{_libdir}
mkdir -p $RPM_BUILD_ROOT/%{_includedir}

pushd build/obj_release
    for file in libtbb{,malloc{,_proxy}}; do
        install -p -D -m 755 ${file}.so.2 $RPM_BUILD_ROOT/%{_libdir}
        ln -s $file.so.2 $RPM_BUILD_ROOT/%{_libdir}/$file.so
    done
popd

pushd include
    find tbb -type f ! -name \*.htm\* -exec \
        install -p -D -m 644 {} $RPM_BUILD_ROOT/%{_includedir}/{} \
    \;
popd

for file in %{SOURCE6} %{SOURCE7} %{SOURCE8}; do
    install -p -D -m 644 $(basename ${file}) \
        $RPM_BUILD_ROOT/%{_libdir}/pkgconfig/$(basename ${file})
done

# Install the rml headers
mkdir -p $RPM_BUILD_ROOT%{_includedir}/rml
cp -p src/rml/include/*.h $RPM_BUILD_ROOT%{_includedir}/rml

# Python 3 install
. build/obj_release/tbbvars.sh
pushd python
%py3_install
chmod a+x $RPM_BUILD_ROOT%{python3_sitearch}/TBB.py
chmod a+x $RPM_BUILD_ROOT%{python3_sitearch}/tbb/__*.py
cp -p libirml.so.1 $RPM_BUILD_ROOT%{_libdir}
ln -s libirml.so.1 $RPM_BUILD_ROOT%{_libdir}/libirml.so
popd

# Install the cmake files
mkdir -p $RPM_BUILD_ROOT%{_libdir}/cmake
cp -a cmake $RPM_BUILD_ROOT%{_libdir}/cmake/%{name}
rm $RPM_BUILD_ROOT%{_libdir}/cmake/%{name}/README.rst

%files -n %{libname}
%doc doc/Release_Notes.txt README.md
%license LICENSE
%{_libdir}/lib*.so.%{libmajor}

%files -n %{irmllib}
%{_libdir}/libirml.so.%{irmlmajor}

%files -n %{devname}
%doc CHANGES cmake/README.rst
%{_includedir}/rml/
%{_includedir}/tbb/
%{_libdir}/cmake/tbb/
%{_libdir}/lib*.so
%{_libdir}/pkgconfig/*.pc

%files doc
%doc doc/Release_Notes.txt
%doc html

%files -n python3-%{name}
%doc python/index.html
%{python3_sitearch}/TBB*
%{python3_sitearch}/tbb/
%{python3_sitearch}/__pycache__/TBB*


%changelog
* Sat Mar 26 2022 umeabot <umeabot> 2020.3-4.mga9
+ Revision: 1827818
- Mageia 9 Mass Rebuild

* Mon Feb 14 2022 tmb <tmb> 2020.3-3.mga9
+ Revision: 1774922
- rebuild for python 3.10

* Tue Mar 23 2021 daviddavid <daviddavid> 2020.3-2.mga9
+ Revision: 1708116
- rebuild for python 3.9

* Tue Sep 01 2020 daviddavid <daviddavid> 2020.3-1.mga8
+ Revision: 1620589
- new version: 2020.3 (sync with Fedora)

* Sun Feb 16 2020 umeabot <umeabot> 2019.2-5.mga8
+ Revision: 1533961
- Mageia 8 Mass Rebuild

* Wed Sep 25 2019 daviddavid <daviddavid> 2019.2-4.mga8
+ Revision: 1445902
- rebuild for python3.8

* Tue Jan 08 2019 tv <tv> 2019.2-3.mga7
+ Revision: 1351598
- rebuild with python 3.7

* Tue Dec 04 2018 wally <wally> 2019.2-2.mga7
+ Revision: 1338320
- new version 2019 update 2 (sync with Fedora)
- drop special SSE2 build for 32-bit x86 as that is now default
- drop python2 support
+ umeabot <umeabot>
- Mageia 7 Mass Rebuild

* Mon Aug 06 2018 wally <wally> 2018.5-1.mga7
+ Revision: 1248345
- new version 2018 Update 5
- update license to ASL 2.0
- sync with Fedora

* Wed Jan 13 2016 pterjan <pterjan> 4.3-1.20140724.5.mga6
+ Revision: 922489
- Steal a patch from Debian to always use GCC atomics

* Sat Aug 22 2015 akien <akien> 4.3-1.20140724.4.mga6
+ Revision: 868028
- Rebuild for gcc 5.2.1

* Wed Oct 15 2014 umeabot <umeabot> 4.3-1.20140724.3.mga6
+ Revision: 741515
- Second Mageia 5 Mass Rebuild

* Tue Sep 16 2014 umeabot <umeabot> 4.3-1.20140724.2.mga5
+ Revision: 689731
- Mageia 5 Mass Rebuild

* Wed Aug 27 2014 akien <akien> 4.3-1.20140724.1.mga5
+ Revision: 668602
- Version 4.3.0

* Sun Jun 15 2014 wally <wally> 4.1-1.20121003.1.mga5
+ Revision: 636629
- fix libification

* Thu Dec 13 2012 shlomif <shlomif> 4.1-1.20121003.mga3
+ Revision: 330401
- New version 4.1-something

* Wed Nov 14 2012 shlomif <shlomif> 4.0-5.20120408.mga3
+ Revision: 317951
- Correct the groups.
- Change to mkrel
- tbb package based on the Fedora package.


* Sat Jul 21 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.0-5.20120408
- Rebuilt for https://fedoraproject.org/wiki/Fedora_18_Mass_Rebuild

* Thu Jun  7 2012 Petr Machata <pmachata@redhat.com> - 4.0-4.20120408
- Rebase to 4.0 update 4
- Refresh Getting_Started.pdf, Reference.pdf, Tutorial.pdf
- Provide pkg-config files
- Resolves: #825402

* Thu Apr 05 2012 Karsten Hopp <karsten@redhat.com> 4.0-3.20110809
- tbb builds now on PPC(64)

* Sat Jan 14 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.0-2.20110809
- Rebuilt for https://fedoraproject.org/wiki/Fedora_17_Mass_Rebuild

* Tue Oct 18 2011 Petr Machata <pmachata@redhat.com> - 4.0-1.20110809
- Rebase to 4.0
  - Port the mfence patch
  - Refresh the documentation bundle

* Tue Jul 26 2011 Petr Machata <pmachata@redhat.com> - 3.0-1.20110419
- Rebase to 3.0-r6
  - Port both patches
  - Package Design_Patterns.pdf
  - Thanks to Richard Shaw for initial rebase patch
- Resolves: #723043

* Wed Feb 09 2011 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.2-3.20090809
- Rebuilt for https://fedoraproject.org/wiki/Fedora_15_Mass_Rebuild

* Thu Jun 10 2010 Petr Machata <pmachata@redhat.com> - 2.2-2.20090809
- Replace mfence instruction with xchg to make it run on ia32-class
  machines without SSE2.
- Resolves: #600654

* Tue Nov  3 2009 Petr Machata <pmachata@redhat.com> - 2.2-1.20090809
- New upstream 2.2
- Resolves: #521571

* Sun Jul 26 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.1-3.20080605
- Rebuilt for https://fedoraproject.org/wiki/Fedora_12_Mass_Rebuild

* Wed Feb 25 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 2.1-2.20080605
- Rebuilt for https://fedoraproject.org/wiki/Fedora_11_Mass_Rebuild

* Fri Jun 13 2008 Petr Machata <pmachata@redhat.com> - 2.1-1.20080605
- New upstream 2.1
  - Drop soname patch, parallel make patch, and GCC 4.3 patch

* Wed Feb 13 2008 Petr Machata <pmachata@redhat.com> - 2.0-4.20070927
- Review fixes
  - Use updated URL
  - More timestamp preservation
- Initial import into Fedora CVS

* Mon Feb 11 2008 Petr Machata <pmachata@redhat.com> - 2.0-3.20070927
- Review fixes
  - Preserve timestamp of installed files
  - Fix soname not to contain "debug"

* Tue Feb  5 2008 Petr Machata <pmachata@redhat.com> - 2.0-2.20070927
- Review fixes
  - GCC 4.3 patchset
  - Add BR util-linux net-tools
  - Add full URL to Source0
  - Build in debug mode to work around problems with GCC 4.3

* Mon Dec 17 2007 Petr Machata <pmachata@redhat.com> - 2.0-1.20070927
- Initial package.
- Using SONAME patch from Debian.
